package com.simuwang.base.common.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.pojo.dos.NavDO;
import com.simuwang.base.pojo.dos.TradeDateDO;
import com.simuwang.base.pojo.dto.EmailFundNavDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class NavDataUtil {

    private static final Logger log = LoggerFactory.getLogger(NavDataUtil.class);

    public static boolean navDataFormatCheck(EmailFundNavDTO fundNavDTO) {
        // 净值日期格式校验
        if (StrUtil.isBlank(fundNavDTO.getPriceDate())) {
            log.warn("净值日期为空 -> 数据:{}", fundNavDTO);
            return false;
        }
        if (StrUtil.isBlank(fundNavDTO.getFundName()) && StrUtil.isBlank(fundNavDTO.getRegisterNumber())) {
            log.warn("备案编码和基金名称都为空-> 数据:{}", fundNavDTO);
            return false;
        }
        if (StrUtil.isNotBlank(fundNavDTO.getFundName()) && fundNavDTO.getFundName().length() > 110) {
            log.warn("基金名称长度超出限制(110个字符)-> 数据:{}", fundNavDTO);
            return false;
        }
        if (StrUtil.isNotBlank(fundNavDTO.getRegisterNumber()) && fundNavDTO.getRegisterNumber().length() > 12) {
            log.warn("备案编码长度超出限制(12个字符)-> 数据:{}", fundNavDTO);
            return false;
        }
        // 单位净值,累计单位净值,资产净值,资产份额数字格式校验
        boolean isvalidNumericFormat = (StrUtil.isBlank(fundNavDTO.getNav()) || StringUtil.isNumeric(fundNavDTO.getNav()))
                && (StrUtil.isBlank(fundNavDTO.getCumulativeNavWithdrawal()) || StringUtil.isNumeric(fundNavDTO.getCumulativeNavWithdrawal()))
                && (StrUtil.isBlank(fundNavDTO.getAssetNet()) || StringUtil.isNumeric(fundNavDTO.getAssetNet()))
                && (StrUtil.isBlank(fundNavDTO.getAssetShare()) || StringUtil.isNumeric(fundNavDTO.getAssetShare()));
        if (!isvalidNumericFormat) {
            log.warn("单位净值或累计净值或资产净值或资产份额格式不正确 -> 数据:{}", fundNavDTO);
            return false;
        }
        log.info("数据格式验证通过 -> {}", fundNavDTO);
        return true;
    }

    public static String checkDataFailReason(EmailFundNavDTO fundNavDTO) {
        // 净值日期格式校验
        if (StrUtil.isBlank(fundNavDTO.getPriceDate())) {
            return "缺少净值日期";
        }
        if (StrUtil.isBlank(fundNavDTO.getFundName()) && StrUtil.isBlank(fundNavDTO.getRegisterNumber())) {
            return "基金名称和备案编码均缺失";
        }
        if(StrUtil.isBlank(fundNavDTO.getNav()) && StrUtil.isBlank(fundNavDTO.getCumulativeNavWithdrawal()) && StrUtil.isBlank(fundNavDTO.getAssetNet())){
            return "单位净值和累计净值和资产净值均缺失";
        }
        // 单位净值,累计单位净值,资产净值,资产份额数字格式校验
        boolean isvalidNumericFormat = (StrUtil.isBlank(fundNavDTO.getNav()) || StringUtil.isNumeric(fundNavDTO.getNav()))
                && (StrUtil.isBlank(fundNavDTO.getCumulativeNavWithdrawal()) || StringUtil.isNumeric(fundNavDTO.getCumulativeNavWithdrawal()))
                && (StrUtil.isBlank(fundNavDTO.getAssetNet()) || StringUtil.isNumeric(fundNavDTO.getAssetNet()))
                && (StrUtil.isBlank(fundNavDTO.getAssetShare()) || StringUtil.isNumeric(fundNavDTO.getAssetShare()));
        if (!isvalidNumericFormat) {
            return "单位净值或累计净值或资产净值或资产份额格式不正确";
        }
        return null;
    }

    public static List<NavDO> filterWeekFrequencyNav(List<NavDO> navDOList, List<TradeDateDO> tradeDateDOList) {
        List<NavDO> dataList = CollUtil.newArrayList();
        if (CollUtil.isEmpty(navDOList)) {
            return dataList;
        }
        Map<Integer, List<TradeDateDO>> yearWeekTradeDateMap = tradeDateDOList.stream().collect(Collectors.groupingBy(TradeDateDO::getYearWeek));

        Map<String, List<NavDO>> fundIdNavDoMap = navDOList.stream().collect(Collectors.groupingBy(NavDO::getFundId));
        for (Map.Entry<String, List<NavDO>> fundIdNavDoEntry : fundIdNavDoMap.entrySet()) {
            List<NavDO> navDoList = fundIdNavDoEntry.getValue();
            for (Map.Entry<Integer, List<TradeDateDO>> yearWeekTradeDateEntry : yearWeekTradeDateMap.entrySet()) {
                List<Date> tradeDateList = yearWeekTradeDateEntry.getValue().stream().map(TradeDateDO::getTradeDate).toList();
                List<NavDO> collect = navDoList.stream().filter(e -> tradeDateList.contains(e.getPriceDate())).toList();
                if (CollUtil.isEmpty(collect)) {
                    continue;
                }
                NavDO navDO = collect.stream().max(Comparator.comparing(NavDO::getPriceDate)).orElse(null);
                dataList.add(navDO);
            }
        }
        return dataList;
    }
}
